package require TclOO
load ../../tclsdl.dll

source map.tcl
source tiled_layer_t.tcl

set ver [SDL_Linked_Version]
set major [SDL_version_major_get $ver]
set minor [SDL_version_minor_get $ver]
set patch [SDL_version_patch_get $ver]
puts "SDL Version: $major.$minor $patch"

set foreground_layer [tiled_layer_t new]
set middle_layer [tiled_layer_t new]
set background_layer [tiled_layer_t new]

proc SDL_LoadBMP {filename} {
	set loadedimage [SDL_LoadBMP_RW [SDL_RWFromFile $filename "r"] 1]
	set optimizedImage [SDL_DisplayFormat $loadedimage]
	SDL_FreeSurface $loadedimage
	return $optimizedImage
}

proc tl_animate {tl dt} {
	$tl set_pos_x [expr [$tl get_pos_x] + double($dt) * [$tl get_vel_x]]
	$tl set_pos_y [expr [$tl get_pos_y] + double($dt) * [$tl get_vel_y]]
}

proc tl_limit_bounce {tl} {
	set maxx [expr 16*48-320]
	set maxy [expr 16*48-240]

	if { [$tl get_pos_x] >= $maxx } {
		$tl set_vel_x [expr -[$tl get_vel_x]]
		$tl set_pos_x [expr $maxx * 2 - [$tl get_pos_x]]

	} elseif { [$tl get_pos_x] <= 0 } {
		$tl set_vel_x [expr -[$tl get_vel_x]]
		$tl set_pos_x [expr -[$tl get_pos_x]]
	
	}
	
	if { [$tl get_pos_y] >= $maxy } {
		$tl set_vel_y [expr -[$tl get_vel_y]]
		$tl set_pos_y [expr $maxy * 2 - [$tl get_pos_y]]
	
	} elseif { [$tl get_pos_y] <= 0 } {
		$tl set_vel_y [expr -[$tl get_vel_y]]
		$tl set_pos_y [expr -[$tl get_pos_y]]
	}
}

proc tl_link { tl to_tl ratio } {
	$tl set_pos_x [expr [$to_tl get_pos_x] * double($ratio)]
	$tl set_pos_y [expr [$to_tl get_pos_y] * double($ratio)]
}

proc ascii_to_int {v} {
	binary scan $v H*@0a* hex ascii
	return [expr int(0x$hex)]
}

proc tl_render { tl screen } {
	set map_x [expr int([$tl get_pos_x]) / 48]
	set map_y [expr int([$tl get_pos_y]) / 48]

	set fine_x [expr int([$tl get_pos_x]) % 48]
	set fine_y [expr int([$tl get_pos_y]) % 48]

	for {set y [expr -$fine_y]} {$y < 240} {incr y 48} {
		set map_x_loop $map_x

		for {set x [expr -$fine_x]} {$x < 320} {incr x 48} {
			set map [$tl get_map]
			set c [string index [lindex $map $map_y] $map_x_loop]
			draw_tile $screen [$tl get_tiles] $x $y $c
			incr map_x_loop
		}

		incr map_y
	}
}

proc draw_tile { screen tiles x y tile } {
	if { $tile == " " } return;

	set tile [ascii_to_int $tile]

	set source_rect [new_SDL_Rect]
	SDL_Rect_x_set $source_rect 0
	SDL_Rect_y_set $source_rect [expr ($tile-[ascii_to_int 0]) * 48]
	SDL_Rect_w_set $source_rect 48
	SDL_Rect_h_set $source_rect 48
	
	set dest_rect [new_SDL_Rect]
	SDL_Rect_x_set $dest_rect $x
	SDL_Rect_y_set $dest_rect $y
	SDL_Rect_w_set $dest_rect 0
	SDL_Rect_h_set $dest_rect 0

	SDL_UpperBlit $tiles $source_rect $screen $dest_rect

	delete_SDL_Rect $source_rect
	delete_SDL_Rect $dest_rect
}

SDL_Init [expr $SDL_INIT_VIDEO|$SDL_INIT_CDROM]
set screen [SDL_SetVideoMode 320 240 16 $SDL_DOUBLEBUF]
SDL_WM_SetCaption "SDL Scrolling Example" ""
set tiles [SDL_LoadBMP "tiles.bmp"]

set flag [expr $SDL_SRCCOLORKEY|$SDL_RLEACCEL]
set format [SDL_Surface_format_get $tiles]
set w [SDL_Surface_w_get $tiles]
set h [SDL_Surface_h_get $tiles]
set rgb [SDL_MapRGB $format 255 0 255]
SDL_SetColorKey $tiles $flag $rgb

$foreground_layer set_pos_x 0.0
$foreground_layer set_pos_y 0.0
$foreground_layer set_vel_x 0.0
$foreground_layer set_vel_y 0.0
$foreground_layer set_map $foreground_map
$foreground_layer set_tiles $tiles

$middle_layer set_pos_x 0.0
$middle_layer set_pos_y 0.0
$middle_layer set_vel_x 0.0
$middle_layer set_vel_y 0.0
$middle_layer set_map $middle_map
$middle_layer set_tiles $tiles

$background_layer set_pos_x 0.0
$background_layer set_pos_y 0.0
$background_layer set_vel_x 0.0
$background_layer set_vel_y 0.0
$background_layer set_map $background_map
$background_layer set_tiles $tiles

$foreground_layer set_vel_x 100.0
$foreground_layer set_vel_y 50.0

set tick1 [SDL_GetTicks]

set event [SDL_Event]
while { [SDL_PollEvent $event] >= 0 } {
	set event_type [SDL_Event_type_get $event] 
    if { $event_type == "$SDL_MOUSEBUTTONDOWN" } {
		break
	}

    if { $event_type == "$SDL_KEYDOWN" } {
        set key_event [SDL_Event_key_get $event]
        set keysym_event [SDL_KeyboardEvent_keysym_get $key_event]
        set key_name [SDL_GetKeyName [SDL_keysym_sym_get $keysym_event]]
        puts "Key: $key_name"
	}
    
    if { $event_type == "$SDL_MOUSEMOTION" } {
        set motion_event [SDL_Event_motion_get $event]
        set x [SDL_MouseMotionEvent_x_get $motion_event]
        set y [SDL_MouseMotionEvent_y_get $motion_event]
        puts "Mouse Motion: $x, $y"
    }

	set tick2 [SDL_GetTicks]
	set dt [expr ($tick2 - $tick1) * double(0.001)]
	set tick1 $tick2

	tl_animate $foreground_layer $dt
	tl_limit_bounce $foreground_layer

	tl_link $middle_layer $foreground_layer 0.5
	tl_link $background_layer $foreground_layer 0.25

	tl_render $background_layer $screen
	tl_render $middle_layer $screen
	tl_render $foreground_layer $screen

	draw_tile $screen $tiles 0 0 4

	SDL_Flip $screen

	SDL_Delay 0
}

SDL_FreeSurface $tiles
